# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from paypalserversdk.api_helper import APIHelper
from paypalserversdk.configuration import Server
from paypalserversdk.http.api_response import ApiResponse
from paypalserversdk.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from paypalserversdk.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from paypalserversdk.models.order import Order
from paypalserversdk.models.order_authorize_response import OrderAuthorizeResponse
from paypalserversdk.exceptions.error_exception import ErrorException


class OrdersController(BaseController):

    """A Controller to access Endpoints in the paypalserversdk API."""
    def __init__(self, config):
        super(OrdersController, self).__init__(config)

    def create_order(self,
                     options=dict()):
        """Does a POST request to /v2/checkout/orders.

        Creates an order. Merchants and partners can add Level 2 and 3 data to
        payments to reduce risk and payment processing costs. For more
        information about processing payments, see checkout or multiparty
        checkout. Note: For error handling and troubleshooting, see Orders v2
        errors.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    body -- OrderRequest -- The request body parameter.
                    paypal_mock_response -- str -- PayPal's REST API uses a
                        request header to invoke negative testing in the
                        sandbox. This header configures the sandbox into a
                        negative testing state for transactions that include
                        the merchant.
                    paypal_request_id -- str -- The server stores keys for 6
                        hours. The API callers can request the times to up to
                        72 hours by speaking to their Account Manager. It is
                        mandatory for all single-step create order calls (E.g.
                        Create Order Request with payment source information
                        like Card, PayPal.vault_id,
                        PayPal.billing_agreement_id, etc).
                    paypal_partner_attribution_id -- str -- The request header
                        parameter.
                    paypal_client_metadata_id -- str -- The request header
                        parameter.
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value is:
                        return=minimal. The server returns a minimal response
                        to optimize communication between the API caller and
                        the server. A minimal response includes the id, status
                        and HATEOAS links. return=representation. The server
                        returns a complete resource representation, including
                        the current state of the resource.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see PayPal-Auth-Assertion.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful response to an idempotent request returns the HTTP
                `200 OK` status code with a JSON response body that shows
                order details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('PayPal-Mock-Response')
                          .value(options.get('paypal_mock_response', None)))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('paypal_request_id', None)))
            .header_param(Parameter()
                          .key('PayPal-Partner-Attribution-Id')
                          .value(options.get('paypal_partner_attribution_id', None)))
            .header_param(Parameter()
                          .key('PayPal-Client-Metadata-Id')
                          .value(options.get('paypal_client_metadata_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Order.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def get_order(self,
                  options=dict()):
        """Does a GET request to /v2/checkout/orders/{id}.

        Shows details for an order, by ID. Note: For error handling and
        troubleshooting, see Orders v2 errors.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order for which to show details.
                    paypal_mock_response -- str -- PayPal's REST API uses a
                        request header to invoke negative testing in the
                        sandbox. This header configures the sandbox into a
                        negative testing state for transactions that include
                        the merchant.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see PayPal-Auth-Assertion.
                    fields -- str -- A comma-separated list of fields that
                        should be returned for the order. Valid filter field
                        is `payment_source`.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `200 OK` status code and a
                JSON response body that shows order details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('PayPal-Mock-Response')
                          .value(options.get('paypal_mock_response', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .query_param(Parameter()
                         .key('fields')
                         .value(options.get('fields', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Order.from_dictionary)
            .is_api_response(True)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def patch_order(self,
                    options=dict()):
        """Does a PATCH request to /v2/checkout/orders/{id}.

        Updates an order with a `CREATED` or `APPROVED` status. You cannot
        update an order with the `COMPLETED` status.<br/><br/>To make an
        update, you must provide a `reference_id`. If you omit this value with
        an order that contains only one purchase unit, PayPal sets the value
        to `default` which enables you to use the path:
        <code>\"/purchase_units/@reference_id=='default'/{attribute-or-object}\
        "</code>. Merchants and partners can add Level 2 and 3 data to
        payments to reduce risk and payment processing costs. For more
        information about processing payments, see <a
        href="https://developer.paypal.com/docs/checkout/advanced/processing/">
        checkout</a> or <a
        href="https://developer.paypal.com/docs/multiparty/checkout/advanced/pr
        ocessing/">multiparty checkout</a>.<blockquote><strong>Note:</strong>
        For error handling and troubleshooting, see <a
        href="https://developer.paypal.com/api/rest/reference/orders/v2/errors/
        #patch-order">Orders v2 errors</a>.</blockquote>Patchable attributes
        or
        objects:<br/><br/><table><thead><th>Attribute</th><th>Op</th><th>Notes<
        /th></thead><tbody><tr><td><code>intent</code></td><td>replace</td><td>
        </td></tr><tr><td><code>payer</code></td><td>replace,
        add</td><td>Using replace op for <code>payer</code> will replace the
        whole <code>payer</code> object with the value sent in
        request.</td></tr><tr><td><code>purchase_units</code></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].custom_id</code></
        td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].description</co
        de></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].payee.email</co
        de></td><td>replace</td><td></td></tr><tr><td><code>purchase_units[].sh
        ipping.name</code></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].shipping.email_add
        ress</code></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].shipping.phone_num
        ber</code></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].shipping.options</
        code></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].shipping.address</
        code></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].shipping.type</cod
        e></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].soft_descriptor</c
        ode></td><td>replace,
        remove</td><td></td></tr><tr><td><code>purchase_units[].amount</code></
        td><td>replace</td><td></td></tr><tr><td><code>purchase_units[].items</
        code></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].invoice_id</cod
        e></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].payment_instruc
        tion</code></td><td>replace</td><td></td></tr><tr><td><code>purchase_un
        its[].payment_instruction.disbursement_mode</code></td><td>replace</td>
        <td>By default, <code>disbursement_mode</code> is
        <code>INSTANT</code>.</td></tr><tr><td><code>purchase_units[].payment_i
        nstruction.payee_receivable_fx_rate_id</code></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].payment_instruc
        tion.platform_fees</code></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].supplementary_d
        ata.airline</code></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].supplementary_d
        ata.card</code></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>application_context.client_confi
        guration</code></td><td>replace, add</td><td></td></tr></tbody></table>

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order to update.
                    paypal_mock_response -- str -- PayPal's REST API uses a
                        request header to invoke negative testing in the
                        sandbox. This header configures the sandbox into a
                        negative testing state for transactions that include
                        the merchant.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see PayPal-Auth-Assertion.
                    body -- List[Patch] -- The request body parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `204 No Content` status
                code with an empty object in the JSON response body.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}')
            .http_method(HttpMethodEnum.PATCH)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Mock-Response')
                          .value(options.get('paypal_mock_response', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def confirm_order(self,
                      options=dict()):
        """Does a POST request to /v2/checkout/orders/{id}/confirm-payment-source.

        Payer confirms their intent to pay for the the Order with the given
        payment source.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order for which the payer
                        confirms their intent to pay.
                    paypal_client_metadata_id -- str -- The request header
                        parameter.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see PayPal-Auth-Assertion.
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value is:
                        return=minimal. The server returns a minimal response
                        to optimize communication between the API caller and
                        the server. A minimal response includes the id, status
                        and HATEOAS links. return=representation. The server
                        returns a complete resource representation, including
                        the current state of the resource.
                    body -- ConfirmOrderRequest -- The request body parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request indicates that the payment source was added
                to the Order. A successful request returns the HTTP `200 OK`
                status code with a JSON response body that shows order details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}/confirm-payment-source')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Client-Metadata-Id')
                          .value(options.get('paypal_client_metadata_id', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Order.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def authorize_order(self,
                        options=dict()):
        """Does a POST request to /v2/checkout/orders/{id}/authorize.

        Authorizes payment for an order. To successfully authorize payment for
        an order, the buyer must first approve the order or a valid
        payment_source must be provided in the request. A buyer can approve
        the order upon being redirected to the rel:approve URL that was
        returned in the HATEOAS links in the create order response. Note: For
        error handling and troubleshooting, see Orders v2 errors.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order for which to authorize.
                    paypal_mock_response -- str -- PayPal's REST API uses a
                        request header to invoke negative testing in the
                        sandbox. This header configures the sandbox into a
                        negative testing state for transactions that include
                        the merchant.
                    paypal_request_id -- str -- The server stores keys for 6
                        hours. The API callers can request the times to up to
                        72 hours by speaking to their Account Manager. It is
                        mandatory for all single-step create order calls (E.g.
                        Create Order Request with payment source information
                        like Card, PayPal.vault_id,
                        PayPal.billing_agreement_id, etc).
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value is:
                        return=minimal. The server returns a minimal response
                        to optimize communication between the API caller and
                        the server. A minimal response includes the id, status
                        and HATEOAS links. return=representation. The server
                        returns a complete resource representation, including
                        the current state of the resource.
                    paypal_client_metadata_id -- str -- The request header
                        parameter.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see PayPal-Auth-Assertion.
                    body -- OrderAuthorizeRequest -- The request body
                        parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful response to an idempotent request returns the HTTP
                `200 OK` status code with a JSON response body that shows
                authorized payment details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}/authorize')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Mock-Response')
                          .value(options.get('paypal_mock_response', None)))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('paypal_request_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('PayPal-Client-Metadata-Id')
                          .value(options.get('paypal_client_metadata_id', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(OrderAuthorizeResponse.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The authorized payment failed due to insufficient permissions.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def capture_order(self,
                      options=dict()):
        """Does a POST request to /v2/checkout/orders/{id}/capture.

        Captures payment for an order. To successfully capture payment for an
        order, the buyer must first approve the order or a valid
        payment_source must be provided in the request. A buyer can approve
        the order upon being redirected to the rel:approve URL that was
        returned in the HATEOAS links in the create order response. Note: For
        error handling and troubleshooting, see Orders v2 errors.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order for which to capture a
                        payment.
                    paypal_mock_response -- str -- PayPal's REST API uses a
                        request header to invoke negative testing in the
                        sandbox. This header configures the sandbox into a
                        negative testing state for transactions that include
                        the merchant.
                    paypal_request_id -- str -- The server stores keys for 6
                        hours. The API callers can request the times to up to
                        72 hours by speaking to their Account Manager. It is
                        mandatory for all single-step create order calls (E.g.
                        Create Order Request with payment source information
                        like Card, PayPal.vault_id,
                        PayPal.billing_agreement_id, etc).
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value is:
                        return=minimal. The server returns a minimal response
                        to optimize communication between the API caller and
                        the server. A minimal response includes the id, status
                        and HATEOAS links. return=representation. The server
                        returns a complete resource representation, including
                        the current state of the resource.
                    paypal_client_metadata_id -- str -- The request header
                        parameter.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see PayPal-Auth-Assertion.
                    body -- OrderCaptureRequest -- The request body parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful response to an idempotent request returns the HTTP
                `200 OK` status code with a JSON response body that shows
                captured payment details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}/capture')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Mock-Response')
                          .value(options.get('paypal_mock_response', None)))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('paypal_request_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('PayPal-Client-Metadata-Id')
                          .value(options.get('paypal_client_metadata_id', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Order.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The authorized payment failed due to insufficient permissions.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def create_order_tracking(self,
                              options=dict()):
        """Does a POST request to /v2/checkout/orders/{id}/track.

        Adds tracking information for an Order.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order that the tracking
                        information is associated with.
                    body -- OrderTrackerRequest -- The request body parameter.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see PayPal-Auth-Assertion.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful response to an idempotent request returns the HTTP
                `200 OK` status code with a JSON response body that shows
                tracker details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}/track')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Order.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def update_order_tracking(self,
                              options=dict()):
        """Does a PATCH request to /v2/checkout/orders/{id}/trackers/{tracker_id}.

        Updates or cancels the tracking information for a PayPal order, by ID.
        Updatable attributes or objects: Attribute Op Notes items replace
        Using replace op for items will replace the entire items object with
        the value sent in request. notify_payer replace, add status replace
        Only patching status to CANCELLED is currently supported.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order that the tracking
                        information is associated with.
                    tracker_id -- str -- The order tracking ID.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see PayPal-Auth-Assertion.
                    body -- List[Patch] -- The request body parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `204 No Content` status
                code with an empty object in the JSON response body.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}/trackers/{tracker_id}')
            .http_method(HttpMethodEnum.PATCH)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .template_param(Parameter()
                            .key('tracker_id')
                            .value(options.get('tracker_id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class VaultCardVerificationMethod(object):

    """Implementation of the 'Vault Card Verification Method' enum.

    The verification method of the card.

    Attributes:
        SCA_WHEN_REQUIRED: When an authorization or transaction is attempted
            this option will return a contingency and HATEOAS link only when
            local regulations require strong customer authentication, (e.g.
            3DS in countries and use cases where it is mandated). The API
            caller should redirect the payer to the link so that they can
            authenticate themselves. In all cases, when an authorization is
            requested the AVS/CVV results will be returned in the response.
        SCA_ALWAYS: Selecting this option will attempt to force a strong
            customer authentication for the authorization/transaction. In
            countries where SCA has been defined and implemented it will
            result in a contingency and HATEOAS link being returned.  The API
            caller should redirect the payer to that link so that they can
            authenticate themselves against their issuing bank or other
            entity. As noted, the HATEOAS link is only available in all
            regions where strong authentication is supported, (e.g. in
            European countries where 3DS is live). Merchants can use this
            setting as an additional layer of security if they choose to. In
            all cases, when an authorization is requested the AVS/CVV results
            will be returned in the response.

    """
    SCA_WHEN_REQUIRED = 'SCA_WHEN_REQUIRED'

    SCA_ALWAYS = 'SCA_ALWAYS'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ExperienceContextShippingPreference(object):

    """Implementation of the 'Experience Context Shipping Preference' enum.

    The location from which the shipping address is derived., The shipping
    preference. This only applies to PayPal payment source., The shipping
    preference. This only applies to PayPal payment source., The location from
    which the shipping address is derived.

    Attributes:
        GET_FROM_FILE: Get the customer-provided shipping address on the
            PayPal site.
        NO_SHIPPING: Redacts the shipping address from the PayPal site.
            Recommended for digital goods.
        SET_PROVIDED_ADDRESS: Merchant sends the shipping address using
            purchase_units.shipping.address. The customer cannot change this
            address on the PayPal site.

    """
    GET_FROM_FILE = 'GET_FROM_FILE'

    NO_SHIPPING = 'NO_SHIPPING'

    SET_PROVIDED_ADDRESS = 'SET_PROVIDED_ADDRESS'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.models.money import Money


class CaptureSubscriptionRequest(object):

    """Implementation of the 'Capture Subscription Request' model.

    The charge amount from the subscriber.

    Attributes:
        note (str): The reason or note for the subscription charge.
        capture_type (CaptureType): The type of capture.
        amount (Money): The currency and amount for a financial transaction,
            such as a balance or payment due.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "note": 'note',
        "capture_type": 'capture_type',
        "amount": 'amount'
    }

    def __init__(self,
                 note=None,
                 capture_type=None,
                 amount=None):
        """Constructor for the CaptureSubscriptionRequest class"""

        # Initialize members of the class
        self.note = note 
        self.capture_type = capture_type 
        self.amount = amount 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        note = dictionary.get("note") if dictionary.get("note") else None
        capture_type = dictionary.get("capture_type") if dictionary.get("capture_type") else None
        amount = Money.from_dictionary(dictionary.get('amount')) if dictionary.get('amount') else None
        # Return an object of this model
        return cls(note,
                   capture_type,
                   amount)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'note={self.note!r}, '
                f'capture_type={self.capture_type!r}, '
                f'amount={self.amount!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'note={self.note!s}, '
                f'capture_type={self.capture_type!s}, '
                f'amount={self.amount!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PaResStatus(object):

    """Implementation of the 'PARes Status' enum.

    Transactions status result identifier. The outcome of the issuer's
    authentication.

    Attributes:
        SUCCESSFULAUTHENTICATION: Successful authentication.
        FAILEDAUTHENTICATION: Failed authentication / account not verified /
            transaction denied.
        UNABLETOCOMPLETEAUTHENTICATION: Unable to complete authentication.
        SUCCESSFULATTEMPTSTRANSACTION: Successful attempts transaction.
        CHALLENGEREQUIRED: Challenge required for authentication.
        AUTHENTICATIONREJECTED: Authentication rejected (merchant must not
            submit for authorization).
        DECOUPLEDAUTHENTICATION: Challenge required; decoupled authentication
            confirmed.
        INFORMATIONALONLY: Informational only; 3DS requestor challenge
            preference acknowledged.

    """
    SUCCESSFULAUTHENTICATION = 'Y'

    FAILEDAUTHENTICATION = 'N'

    UNABLETOCOMPLETEAUTHENTICATION = 'U'

    SUCCESSFULATTEMPTSTRANSACTION = 'A'

    CHALLENGEREQUIRED = 'C'

    AUTHENTICATIONREJECTED = 'R'

    DECOUPLEDAUTHENTICATION = 'D'

    INFORMATIONALONLY = 'I'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class CaptureStatus(object):

    """Implementation of the 'Capture Status' enum.

    The status of the captured payment.

    Attributes:
        COMPLETED: The funds for this captured payment were credited to the
            payee's PayPal account.
        DECLINED: The funds could not be captured.
        PARTIALLY_REFUNDED: An amount less than this captured payment's amount
            was partially refunded to the payer.
        PENDING: The funds for this captured payment was not yet credited to
            the payee's PayPal account. For more information, see
            status.details.
        REFUNDED: An amount greater than or equal to this captured payment's
            amount was refunded to the payer.
        FAILED: There was an error while capturing payment.

    """
    COMPLETED = 'COMPLETED'

    DECLINED = 'DECLINED'

    PARTIALLY_REFUNDED = 'PARTIALLY_REFUNDED'

    PENDING = 'PENDING'

    REFUNDED = 'REFUNDED'

    FAILED = 'FAILED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class SubscriptionsCardBrand(object):

    """Implementation of the 'Subscriptions Card Brand' enum.

    The card network or brand. Applies to credit, debit, gift, and payment
    cards.

    Attributes:
        VISA: Visa card.
        MASTERCARD: Mastercard card.
        DISCOVER: Discover card.
        AMEX: American Express card.
        SOLO: Solo debit card.
        JCB: Japan Credit Bureau card.
        STAR: Military Star card.
        DELTA: Delta Airlines card.
        SWITCH: Switch credit card.
        MAESTRO: Maestro credit card.
        CB_NATIONALE: Carte Bancaire (CB) credit card.
        CONFIGOGA: Configoga credit card.
        CONFIDIS: Confidis credit card.
        ELECTRON: Visa Electron credit card.
        CETELEM: Cetelem credit card.
        CHINA_UNION_PAY: China union pay credit card.
        DINERS: The Diners Club International banking and payment services
            capability network owned by Discover Financial Services (DFS), one
            of the most recognized brands in US financial services.
        ELO: The Brazilian Elo card payment network.
        HIPER: The Hiper - Ingenico ePayment network.
        HIPERCARD: The Brazilian Hipercard payment network that's widely
            accepted in the retail market.
        RUPAY: The RuPay payment network.
        GE: The GE Credit Union 3Point card payment network.
        SYNCHRONY: The Synchrony Financial (SYF) payment network.
        EFTPOS: The Electronic Fund Transfer At Point of Sale(EFTPOS) Debit
            card payment network.
        UNKNOWN: UNKNOWN payment network.

    """
    VISA = 'VISA'

    MASTERCARD = 'MASTERCARD'

    DISCOVER = 'DISCOVER'

    AMEX = 'AMEX'

    SOLO = 'SOLO'

    JCB = 'JCB'

    STAR = 'STAR'

    DELTA = 'DELTA'

    SWITCH = 'SWITCH'

    MAESTRO = 'MAESTRO'

    CB_NATIONALE = 'CB_NATIONALE'

    CONFIGOGA = 'CONFIGOGA'

    CONFIDIS = 'CONFIDIS'

    ELECTRON = 'ELECTRON'

    CETELEM = 'CETELEM'

    CHINA_UNION_PAY = 'CHINA_UNION_PAY'

    DINERS = 'DINERS'

    ELO = 'ELO'

    HIPER = 'HIPER'

    HIPERCARD = 'HIPERCARD'

    RUPAY = 'RUPAY'

    GE = 'GE'

    SYNCHRONY = 'SYNCHRONY'

    EFTPOS = 'EFTPOS'

    UNKNOWN = 'UNKNOWN'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class Money(object):

    """Implementation of the 'Money' model.

    The currency and amount for a financial transaction, such as a balance or
    payment due.

    Attributes:
        currency_code (str): The [three-character ISO-4217 currency
            code](/api/rest/reference/currency-codes/) that identifies the
            currency.
        value (str): The value, which might be: An integer for currencies like
            `JPY` that are not typically fractional. A decimal fraction for
            currencies like `TND` that are subdivided into thousandths. For
            the required number of decimal places for a currency code, see
            [Currency Codes](/api/rest/reference/currency-codes/).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "currency_code": 'currency_code',
        "value": 'value'
    }

    def __init__(self,
                 currency_code=None,
                 value=None):
        """Constructor for the Money class"""

        # Initialize members of the class
        self.currency_code = currency_code 
        self.value = value 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        currency_code = dictionary.get("currency_code") if dictionary.get("currency_code") else None
        value = dictionary.get("value") if dictionary.get("value") else None
        # Return an object of this model
        return cls(currency_code,
                   value)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'currency_code={self.currency_code!r}, '
                f'value={self.value!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'currency_code={self.currency_code!s}, '
                f'value={self.value!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.pricing_scheme import PricingScheme


class BillingCycle(object):

    """Implementation of the 'Billing Cycle' model.

    The billing cycle providing details of the billing frequency, amount,
    duration and if the billing cycle is a free, discounted or regular billing
    cycle. The sequence of the billing cycle will be in the following order -
    free trial billing cycle(s), discounted trial billing cycle(s), regular
    billing cycle(s).

    Attributes:
        tenure_type (TenureType): The tenure type of the billing cycle
            identifies if the billing cycle is a trial(free or discounted) or
            regular billing cycle.
        pricing_scheme (PricingScheme): The pricing scheme details.
        total_cycles (int): The number of times this billing cycle gets
            executed. Trial billing cycles can only be executed a finite
            number of times (value between 1 and 999 for total_cycles).
            Regular billing cycles can be executed infinite times (value of 0
            for total_cycles) or a finite number of times (value between 1 and
            999 for total_cycles).
        sequence (int): The order in which this cycle is to run among other
            billing cycles. For example, a trial billing cycle has a
            `sequence` of `1` while a regular billing cycle has a `sequence`
            of `2`, so that trial cycle runs before the regular cycle.
        start_date (str): The stand-alone date, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). To
            represent special legal values, such as a date of birth, you
            should use dates with no associated time or time-zone data.
            Whenever possible, use the standard `date_time` type. This regular
            expression does not validate all dates. For example, February 31
            is valid and nothing is known about leap years.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "tenure_type": 'tenure_type',
        "pricing_scheme": 'pricing_scheme',
        "total_cycles": 'total_cycles',
        "sequence": 'sequence',
        "start_date": 'start_date'
    }

    _optionals = [
        'pricing_scheme',
        'total_cycles',
        'sequence',
        'start_date',
    ]

    def __init__(self,
                 tenure_type=None,
                 pricing_scheme=APIHelper.SKIP,
                 total_cycles=1,
                 sequence=1,
                 start_date=APIHelper.SKIP):
        """Constructor for the BillingCycle class"""

        # Initialize members of the class
        self.tenure_type = tenure_type 
        if pricing_scheme is not APIHelper.SKIP:
            self.pricing_scheme = pricing_scheme 
        self.total_cycles = total_cycles 
        self.sequence = sequence 
        if start_date is not APIHelper.SKIP:
            self.start_date = start_date 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        tenure_type = dictionary.get("tenure_type") if dictionary.get("tenure_type") else None
        pricing_scheme = PricingScheme.from_dictionary(dictionary.get('pricing_scheme')) if 'pricing_scheme' in dictionary.keys() else APIHelper.SKIP
        total_cycles = dictionary.get("total_cycles") if dictionary.get("total_cycles") else 1
        sequence = dictionary.get("sequence") if dictionary.get("sequence") else 1
        start_date = dictionary.get("start_date") if dictionary.get("start_date") else APIHelper.SKIP
        # Return an object of this model
        return cls(tenure_type,
                   pricing_scheme,
                   total_cycles,
                   sequence,
                   start_date)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'tenure_type={self.tenure_type!r}, '
                f'pricing_scheme={(self.pricing_scheme if hasattr(self, "pricing_scheme") else None)!r}, '
                f'total_cycles={(self.total_cycles if hasattr(self, "total_cycles") else None)!r}, '
                f'sequence={(self.sequence if hasattr(self, "sequence") else None)!r}, '
                f'start_date={(self.start_date if hasattr(self, "start_date") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'tenure_type={self.tenure_type!s}, '
                f'pricing_scheme={(self.pricing_scheme if hasattr(self, "pricing_scheme") else None)!s}, '
                f'total_cycles={(self.total_cycles if hasattr(self, "total_cycles") else None)!s}, '
                f'sequence={(self.sequence if hasattr(self, "sequence") else None)!s}, '
                f'start_date={(self.start_date if hasattr(self, "start_date") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class OrderApplicationContextShippingPreference(object):

    """Implementation of the 'Order Application Context Shipping Preference' enum.

    DEPRECATED. DEPRECATED. The shipping preference: Displays the shipping
    address to the customer. Enables the customer to choose an address on the
    PayPal site. Restricts the customer from changing the address during the
    payment-approval process. .  The fields in `application_context` are now
    available in the `experience_context` object under the `payment_source`
    which supports them (eg.
    `payment_source.paypal.experience_context.shipping_preference`). Please
    specify this field in the `experience_context` object instead of the
    `application_context` object.

    Attributes:
        GET_FROM_FILE: Use the customer-provided shipping address on the
            PayPal site.
        NO_SHIPPING: Redact the shipping address from the PayPal site.
            Recommended for digital goods.
        SET_PROVIDED_ADDRESS: Use the merchant-provided address. The customer
            cannot change this address on the PayPal site.

    """
    GET_FROM_FILE = 'GET_FROM_FILE'

    NO_SHIPPING = 'NO_SHIPPING'

    SET_PROVIDED_ADDRESS = 'SET_PROVIDED_ADDRESS'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.bank_request import BankRequest
from paypalserversdk.models.setup_token_request_card import SetupTokenRequestCard
from paypalserversdk.models.vault_apple_pay_request import VaultApplePayRequest
from paypalserversdk.models.vault_paypal_wallet_request import VaultPaypalWalletRequest
from paypalserversdk.models.vault_token_request import VaultTokenRequest
from paypalserversdk.models.vault_venmo_request import VaultVenmoRequest


class SetupTokenRequestPaymentSource(object):

    """Implementation of the 'Setup Token Request Payment Source' model.

    The payment method to vault with the instrument details.

    Attributes:
        card (SetupTokenRequestCard): A Resource representing a request to
            vault a Card.
        paypal (VaultPaypalWalletRequest): A resource representing a request
            to vault PayPal Wallet.
        venmo (VaultVenmoRequest): A resource representing a request to vault
            Venmo.
        apple_pay (VaultApplePayRequest): A resource representing a request to
            vault Apple Pay.
        token (VaultTokenRequest): The Tokenized Payment Source representing a
            Request to Vault a Token.
        bank (BankRequest): A Resource representing a request to vault a Bank
            used for ACH Debit.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "card": 'card',
        "paypal": 'paypal',
        "venmo": 'venmo',
        "apple_pay": 'apple_pay',
        "token": 'token',
        "bank": 'bank'
    }

    _optionals = [
        'card',
        'paypal',
        'venmo',
        'apple_pay',
        'token',
        'bank',
    ]

    def __init__(self,
                 card=APIHelper.SKIP,
                 paypal=APIHelper.SKIP,
                 venmo=APIHelper.SKIP,
                 apple_pay=APIHelper.SKIP,
                 token=APIHelper.SKIP,
                 bank=APIHelper.SKIP):
        """Constructor for the SetupTokenRequestPaymentSource class"""

        # Initialize members of the class
        if card is not APIHelper.SKIP:
            self.card = card 
        if paypal is not APIHelper.SKIP:
            self.paypal = paypal 
        if venmo is not APIHelper.SKIP:
            self.venmo = venmo 
        if apple_pay is not APIHelper.SKIP:
            self.apple_pay = apple_pay 
        if token is not APIHelper.SKIP:
            self.token = token 
        if bank is not APIHelper.SKIP:
            self.bank = bank 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        card = SetupTokenRequestCard.from_dictionary(dictionary.get('card')) if 'card' in dictionary.keys() else APIHelper.SKIP
        paypal = VaultPaypalWalletRequest.from_dictionary(dictionary.get('paypal')) if 'paypal' in dictionary.keys() else APIHelper.SKIP
        venmo = VaultVenmoRequest.from_dictionary(dictionary.get('venmo')) if 'venmo' in dictionary.keys() else APIHelper.SKIP
        apple_pay = VaultApplePayRequest.from_dictionary(dictionary.get('apple_pay')) if 'apple_pay' in dictionary.keys() else APIHelper.SKIP
        token = VaultTokenRequest.from_dictionary(dictionary.get('token')) if 'token' in dictionary.keys() else APIHelper.SKIP
        bank = BankRequest.from_dictionary(dictionary.get('bank')) if 'bank' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(card,
                   paypal,
                   venmo,
                   apple_pay,
                   token,
                   bank)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'card={(self.card if hasattr(self, "card") else None)!r}, '
                f'paypal={(self.paypal if hasattr(self, "paypal") else None)!r}, '
                f'venmo={(self.venmo if hasattr(self, "venmo") else None)!r}, '
                f'apple_pay={(self.apple_pay if hasattr(self, "apple_pay") else None)!r}, '
                f'token={(self.token if hasattr(self, "token") else None)!r}, '
                f'bank={(self.bank if hasattr(self, "bank") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'card={(self.card if hasattr(self, "card") else None)!s}, '
                f'paypal={(self.paypal if hasattr(self, "paypal") else None)!s}, '
                f'venmo={(self.venmo if hasattr(self, "venmo") else None)!s}, '
                f'apple_pay={(self.apple_pay if hasattr(self, "apple_pay") else None)!s}, '
                f'token={(self.token if hasattr(self, "token") else None)!s}, '
                f'bank={(self.bank if hasattr(self, "bank") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class VaultInstructionAction(object):

    """Implementation of the 'Vault Instruction Action' enum.

    Vault Instruction on action to be performed after a successful payer
    approval.

    Attributes:
        ON_CREATE_PAYMENT_TOKENS: Vault the payment method after API caller
            performs a successful POST on Payment Tokens.
        ON_PAYER_APPROVAL: Vault the payment method on successful payer
            authentication and approval.

    """
    ON_CREATE_PAYMENT_TOKENS = 'ON_CREATE_PAYMENT_TOKENS'

    ON_PAYER_APPROVAL = 'ON_PAYER_APPROVAL'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.billing_cycle_override import BillingCycleOverride
from paypalserversdk.models.payment_preferences_override import PaymentPreferencesOverride
from paypalserversdk.models.taxes_override import TaxesOverride


class PlanOverride(object):

    """Implementation of the 'Plan Override' model.

    An inline plan object to customise the subscription. You can override plan
    level default attributes by providing customised values for the
    subscription in this object.

    Attributes:
        billing_cycles (List[BillingCycleOverride]): An array of billing
            cycles for trial billing and regular billing. The subscription
            billing cycle definition has to adhere to the plan billing cycle
            definition.
        payment_preferences (PaymentPreferencesOverride): The payment
            preferences to override at subscription level.
        taxes (TaxesOverride): The tax details.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "billing_cycles": 'billing_cycles',
        "payment_preferences": 'payment_preferences',
        "taxes": 'taxes'
    }

    _optionals = [
        'billing_cycles',
        'payment_preferences',
        'taxes',
    ]

    def __init__(self,
                 billing_cycles=APIHelper.SKIP,
                 payment_preferences=APIHelper.SKIP,
                 taxes=APIHelper.SKIP):
        """Constructor for the PlanOverride class"""

        # Initialize members of the class
        if billing_cycles is not APIHelper.SKIP:
            self.billing_cycles = billing_cycles 
        if payment_preferences is not APIHelper.SKIP:
            self.payment_preferences = payment_preferences 
        if taxes is not APIHelper.SKIP:
            self.taxes = taxes 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        billing_cycles = None
        if dictionary.get('billing_cycles') is not None:
            billing_cycles = [BillingCycleOverride.from_dictionary(x) for x in dictionary.get('billing_cycles')]
        else:
            billing_cycles = APIHelper.SKIP
        payment_preferences = PaymentPreferencesOverride.from_dictionary(dictionary.get('payment_preferences')) if 'payment_preferences' in dictionary.keys() else APIHelper.SKIP
        taxes = TaxesOverride.from_dictionary(dictionary.get('taxes')) if 'taxes' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(billing_cycles,
                   payment_preferences,
                   taxes)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'billing_cycles={(self.billing_cycles if hasattr(self, "billing_cycles") else None)!r}, '
                f'payment_preferences={(self.payment_preferences if hasattr(self, "payment_preferences") else None)!r}, '
                f'taxes={(self.taxes if hasattr(self, "taxes") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'billing_cycles={(self.billing_cycles if hasattr(self, "billing_cycles") else None)!s}, '
                f'payment_preferences={(self.payment_preferences if hasattr(self, "payment_preferences") else None)!s}, '
                f'taxes={(self.taxes if hasattr(self, "taxes") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class AuthorizationStatus(object):

    """Implementation of the 'Authorization Status' enum.

    The status for the authorized payment.

    Attributes:
        CREATED: The authorized payment is created. No captured payments have
            been made for this authorized payment.
        CAPTURED: The authorized payment has one or more captures against it.
            The sum of these captured payments is greater than the amount of
            the original authorized payment.
        DENIED: PayPal cannot authorize funds for this authorized payment.
        PARTIALLY_CAPTURED: A captured payment was made for the authorized
            payment for an amount that is less than the amount of the original
            authorized payment.
        VOIDED: The authorized payment was voided. No more captured payments
            can be made against this authorized payment.
        PENDING: The created authorization is in pending state. For more
            information, see status.details.

    """
    CREATED = 'CREATED'

    CAPTURED = 'CAPTURED'

    DENIED = 'DENIED'

    PARTIALLY_CAPTURED = 'PARTIALLY_CAPTURED'

    VOIDED = 'VOIDED'

    PENDING = 'PENDING'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.simple_postal_address_coarse_grained import SimplePostalAddressCoarseGrained


class ShippingInformation(object):

    """Implementation of the 'Shipping Information' model.

    The shipping information.

    Attributes:
        name (str): The recipient's name.
        method (str): The shipping method that is associated with this order.
        address (SimplePostalAddressCoarseGrained): A simple postal address
            with coarse-grained fields. Do not use for an international
            address. Use for backward compatibility only. Does not contain
            phone.
        secondary_shipping_address (SimplePostalAddressCoarseGrained): A
            simple postal address with coarse-grained fields. Do not use for
            an international address. Use for backward compatibility only.
            Does not contain phone.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name": 'name',
        "method": 'method',
        "address": 'address',
        "secondary_shipping_address": 'secondary_shipping_address'
    }

    _optionals = [
        'name',
        'method',
        'address',
        'secondary_shipping_address',
    ]

    def __init__(self,
                 name=APIHelper.SKIP,
                 method=APIHelper.SKIP,
                 address=APIHelper.SKIP,
                 secondary_shipping_address=APIHelper.SKIP):
        """Constructor for the ShippingInformation class"""

        # Initialize members of the class
        if name is not APIHelper.SKIP:
            self.name = name 
        if method is not APIHelper.SKIP:
            self.method = method 
        if address is not APIHelper.SKIP:
            self.address = address 
        if secondary_shipping_address is not APIHelper.SKIP:
            self.secondary_shipping_address = secondary_shipping_address 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        method = dictionary.get("method") if dictionary.get("method") else APIHelper.SKIP
        address = SimplePostalAddressCoarseGrained.from_dictionary(dictionary.get('address')) if 'address' in dictionary.keys() else APIHelper.SKIP
        secondary_shipping_address = SimplePostalAddressCoarseGrained.from_dictionary(dictionary.get('secondary_shipping_address')) if 'secondary_shipping_address' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(name,
                   method,
                   address,
                   secondary_shipping_address)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'method={(self.method if hasattr(self, "method") else None)!r}, '
                f'address={(self.address if hasattr(self, "address") else None)!r}, '
                f'secondary_shipping_address={(self.secondary_shipping_address if hasattr(self, "secondary_shipping_address") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'method={(self.method if hasattr(self, "method") else None)!s}, '
                f'address={(self.address if hasattr(self, "address") else None)!s}, '
                f'secondary_shipping_address={(self.secondary_shipping_address if hasattr(self, "secondary_shipping_address") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class VaultTokenRequestType(object):

    """Implementation of the 'Vault Token Request Type' enum.

    The tokenization method that generated the ID.

    Attributes:
        SETUP_TOKEN: The setup token, which is a temporary reference to
            payment source.

    """
    SETUP_TOKEN = 'SETUP_TOKEN'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from paypalserversdk.api_helper import APIHelper
from paypalserversdk.configuration import Server
from paypalserversdk.http.api_response import ApiResponse
from paypalserversdk.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from paypalserversdk.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from paypalserversdk.models.billing_plan import BillingPlan
from paypalserversdk.models.plan_collection import PlanCollection
from paypalserversdk.models.subscription import Subscription
from paypalserversdk.models.subscription_collection import SubscriptionCollection
from paypalserversdk.models.modify_subscription_response import ModifySubscriptionResponse
from paypalserversdk.models.subscription_transaction_details import SubscriptionTransactionDetails
from paypalserversdk.models.transactions_list import TransactionsList
from paypalserversdk.exceptions.subscription_error_exception import SubscriptionErrorException


class SubscriptionsController(BaseController):

    """A Controller to access Endpoints in the paypalserversdk API."""
    def __init__(self, config):
        super(SubscriptionsController, self).__init__(config)

    def create_billing_plan(self,
                            options=dict()):
        """Does a POST request to /v1/billing/plans.

        Creates a plan that defines pricing and billing cycle details for
        subscriptions.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value is:
                        return=minimal. The server returns a minimal response
                        to optimize communication between the API caller and
                        the server. A minimal response includes the id, status
                        and HATEOAS links. return=representation. The server
                        returns a complete resource representation, including
                        the current state of the resource.
                    paypal_request_id -- str -- The server stores keys for 72
                        hours.
                    body -- PlanRequest -- The request body parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `200 OK` status code and a
                JSON response body that shows billing plan details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/plans')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('paypal_request_id', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(BillingPlan.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Bad Request. Request is not well-formed, syntactically incorrect, or violates schema.', SubscriptionErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def list_billing_plans(self,
                           options=dict()):
        """Does a GET request to /v1/billing/plans.

        Lists billing plans.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value is:
                        return=minimal. The server returns a minimal response
                        to optimize communication between the API caller and
                        the server. A minimal response includes the id, name,
                        description and HATEOAS links. return=representation.
                        The server returns a complete resource representation,
                        including the current state of the resource.
                    product_id -- str -- Filters the response by a Product ID.
                    page_size -- int -- The number of items to return in the
                        response.
                    page -- int -- A non-zero integer which is the start index
                        of the entire list of items to return in the response.
                        The combination of `page=1` and `page_size=20` returns
                        the first 20 items. The combination of `page=2` and
                        `page_size=20` returns the next 20 items.
                    total_required -- bool -- Indicates whether to show the
                        total count in the response.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `200 OK` status code and a
                JSON response body that lists billing plans.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/plans')
            .http_method(HttpMethodEnum.GET)
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .query_param(Parameter()
                         .key('product_id')
                         .value(options.get('product_id', None)))
            .query_param(Parameter()
                         .key('page_size')
                         .value(options.get('page_size', None)))
            .query_param(Parameter()
                         .key('page')
                         .value(options.get('page', None)))
            .query_param(Parameter()
                         .key('total_required')
                         .value(options.get('total_required', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PlanCollection.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', SubscriptionErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def get_billing_plan(self,
                         id):
        """Does a GET request to /v1/billing/plans/{id}.

        Shows details for a plan, by ID.

        Args:
            id (str): The ID of the plan.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `200 OK` status code and a
                JSON response body that shows plan details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/plans/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(BillingPlan.from_dictionary)
            .is_api_response(True)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def patch_billing_plan(self,
                           options=dict()):
        """Does a PATCH request to /v1/billing/plans/{id}.

        Updates a plan with the `CREATED` or `ACTIVE` status. For an
        `INACTIVE` plan, you can make only status updates. You can patch these
        attributes and objects: Attribute or object Operations description
        replace payment_preferences.auto_bill_outstanding replace
        taxes.percentage replace payment_preferences.payment_failure_threshold
        replace payment_preferences.setup_fee replace
        payment_preferences.setup_fee_failure_action replace name replace

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the plan.
                    body -- List[Patch] -- The request body parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `204 No Content` status
                code with no JSON response body.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/plans/{id}')
            .http_method(HttpMethodEnum.PATCH)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', SubscriptionErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def activate_billing_plan(self,
                              id):
        """Does a POST request to /v1/billing/plans/{id}/activate.

        Activates a plan, by ID.

        Args:
            id (str): The ID of the plan.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `204 No Content` status
                code with no JSON response body.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/plans/{id}/activate')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .is_api_response(True)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def deactivate_billing_plan(self,
                                id):
        """Does a POST request to /v1/billing/plans/{id}/deactivate.

        Deactivates a plan, by ID.

        Args:
            id (str): The ID of the plan.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `204 No Content` status
                code with no JSON response body.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/plans/{id}/deactivate')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .is_api_response(True)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def update_billing_plan_pricing_schemes(self,
                                            options=dict()):
        """Does a POST request to /v1/billing/plans/{id}/update-pricing-schemes.

        Updates pricing for a plan. For example, you can update a regular
        billing cycle from $5 per month to $7 per month.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID for the plan.
                    body -- UpdatePricingSchemesRequest -- The request body
                        parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `204 No Content` status
                code with no JSON response body.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/plans/{id}/update-pricing-schemes')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .is_api_response(True)
            .local_error('400', 'Bad Request. Request is not well-formed, syntactically incorrect, or violates schema.', SubscriptionErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def create_subscription(self,
                            options=dict()):
        """Does a POST request to /v1/billing/subscriptions.

        Creates a subscription.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value is:
                        return=minimal. The server returns a minimal response
                        to optimize communication between the API caller and
                        the server. A minimal response includes the id, status
                        and HATEOAS links. return=representation. The server
                        returns a complete resource representation, including
                        the current state of the resource.
                    paypal_request_id -- str -- The server stores keys for 72
                        hours.
                    paypal_client_metadata_id -- str -- The PayPal Client
                        Metadata Id(CMID) is used to provide device-specific
                        information to PayPal's risk engine. This is crucial
                        for transactions that require device-specific risk
                        assessments. Merchants typically use the Paypal SDK
                        that automatically submits the CMID or they use tools
                        like Fraudnet JS for web or Magnes JS for mobile to
                        generate the CMID on the frontend and then pass it to
                        the API as part of the request headers.
                    body -- CreateSubscriptionRequest -- The request body
                        parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `200 OK` status code and a
                JSON response body that shows subscription details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/subscriptions')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('paypal_request_id', None)))
            .header_param(Parameter()
                          .key('PayPal-Client-Metadata-Id')
                          .value(options.get('paypal_client_metadata_id', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Subscription.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Bad Request. Request is not well-formed, syntactically incorrect, or violates schema.', SubscriptionErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def list_subscriptions(self,
                           options=dict()):
        """Does a GET request to /v1/billing/subscriptions.

        List all subscriptions for merchant account.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    plan_ids -- str -- Filters the response by list of plan
                        IDs. Filter supports upto 70 plan IDs. URLs should not
                        exceed a length of 2000 characters.
                    statuses -- str -- Filters the response by list of
                        subscription statuses.
                    created_after -- str -- Filters the response by
                        subscription creation start time for a range of
                        subscriptions.
                    created_before -- str -- Filters the response by
                        subscription creation end time for a range of
                        subscriptions.
                    status_updated_before -- str -- Filters the response by
                        status update start time for a range of subscriptions.
                    status_updated_after -- str -- Filters the response by
                        status update end time for a range of subscriptions.
                    filter -- str -- Filter the response using complex
                        expressions that could use comparison operators like
                        ge, gt, le, lt and logical operators such as 'and' and
                        'or'.
                    page_size -- int -- The number of items to return in the
                        response.
                    page -- int -- A non-zero integer which is the start index
                        of the entire list of items to return in the response.
                        The combination of `page=1` and `page_size=20` returns
                        the first 20 items. The combination of `page=2` and
                        `page_size=20` returns the next 20 items.
                    customer_ids -- List[str] -- Filters the response by comma
                        separated vault customer IDs (FSS subscriptions only).

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `200 OK` status code and a
                JSON response body that lists the subscriptions.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/subscriptions')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('plan_ids')
                         .value(options.get('plan_ids', None)))
            .query_param(Parameter()
                         .key('statuses')
                         .value(options.get('statuses', None)))
            .query_param(Parameter()
                         .key('created_after')
                         .value(options.get('created_after', None)))
            .query_param(Parameter()
                         .key('created_before')
                         .value(options.get('created_before', None)))
            .query_param(Parameter()
                         .key('status_updated_before')
                         .value(options.get('status_updated_before', None)))
            .query_param(Parameter()
                         .key('status_updated_after')
                         .value(options.get('status_updated_after', None)))
            .query_param(Parameter()
                         .key('filter')
                         .value(options.get('filter', None)))
            .query_param(Parameter()
                         .key('page_size')
                         .value(options.get('page_size', None)))
            .query_param(Parameter()
                         .key('page')
                         .value(options.get('page', None)))
            .query_param(Parameter()
                         .key('customer_ids')
                         .value(options.get('customer_ids', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(SubscriptionCollection.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', SubscriptionErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def get_subscription(self,
                         options=dict()):
        """Does a GET request to /v1/billing/subscriptions/{id}.

        Shows details for a subscription, by ID.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the subscription.
                    fields -- str -- List of fields that are to be returned in
                        the response. Possible value for fields are
                        last_failed_payment and plan.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `200 OK` status code and a
                JSON response body that shows subscription details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/subscriptions/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .query_param(Parameter()
                         .key('fields')
                         .value(options.get('fields', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Subscription.from_dictionary)
            .is_api_response(True)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def patch_subscription(self,
                           options=dict()):
        """Does a PATCH request to /v1/billing/subscriptions/{id}.

        Updates a subscription which could be in ACTIVE or SUSPENDED status.
        You can override plan level default attributes by providing customised
        values for plan path in the patch request. You cannot update
        attributes that have already completed (Example - trial cycles can’t
        be updated if completed). Once overridden, changes to plan resource
        will not impact subscription. Any price update will not impact billing
        cycles within next 10 days (Applicable only for subscriptions funded
        by PayPal account). Following are the fields eligible for patch.
        Attribute or object Operations billing_info.outstanding_balance
        replace custom_id add,replace plan.billing_cycles[@sequence==n].
        pricing_scheme.fixed_price add,replace
        plan.billing_cycles[@sequence==n]. pricing_scheme.tiers replace
        plan.billing_cycles[@sequence==n]. total_cycles replace
        plan.payment_preferences. auto_bill_outstanding replace
        plan.payment_preferences. payment_failure_threshold replace
        plan.taxes.inclusive add,replace plan.taxes.percentage add,replace
        shipping_amount add,replace start_time replace
        subscriber.shipping_address add,replace subscriber.payment_source (for
        subscriptions funded by card payments) replace

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID for the subscription.
                    body -- List[Patch] -- The request body parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `204 No Content` status
                code with no JSON response body.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/subscriptions/{id}')
            .http_method(HttpMethodEnum.PATCH)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', SubscriptionErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def revise_subscription(self,
                            options=dict()):
        """Does a POST request to /v1/billing/subscriptions/{id}/revise.

        Updates the quantity of the product or service in a subscription. You
        can also use this method to switch the plan and update the
        `shipping_amount`, `shipping_address` values for the subscription.
        This type of update requires the buyer's consent.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the subscription.
                    body -- ModifySubscriptionRequest -- The request body
                        parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `200 OK` status code and a
                JSON response body that shows subscription details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/subscriptions/{id}/revise')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ModifySubscriptionResponse.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Bad Request. Request is not well-formed, syntactically incorrect, or violates schema.', SubscriptionErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def suspend_subscription(self,
                             options=dict()):
        """Does a POST request to /v1/billing/subscriptions/{id}/suspend.

        Suspends the subscription.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the subscription.
                    body -- SuspendSubscription -- The request body parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `204 No Content` status
                code with no JSON response body.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/subscriptions/{id}/suspend')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .is_api_response(True)
            .local_error('400', 'Bad Request. Request is not well-formed, syntactically incorrect, or violates schema.', SubscriptionErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def cancel_subscription(self,
                            options=dict()):
        """Does a POST request to /v1/billing/subscriptions/{id}/cancel.

        Cancels the subscription.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the subscription.
                    body -- CancelSubscriptionRequest -- The request body
                        parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `204 No Content` status
                code with no JSON response body.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/subscriptions/{id}/cancel')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .is_api_response(True)
            .local_error('400', 'Bad Request. Request is not well-formed, syntactically incorrect, or violates schema.', SubscriptionErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def activate_subscription(self,
                              options=dict()):
        """Does a POST request to /v1/billing/subscriptions/{id}/activate.

        Activates the subscription.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the subscription.
                    body -- ActivateSubscriptionRequest -- The request body
                        parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `204 No Content` status
                code with no JSON response body.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/subscriptions/{id}/activate')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .is_api_response(True)
            .local_error('400', 'Bad Request. Request is not well-formed, syntactically incorrect, or violates schema.', SubscriptionErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def capture_subscription(self,
                             options=dict()):
        """Does a POST request to /v1/billing/subscriptions/{id}/capture.

        Captures an authorized payment from the subscriber on the subscription.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the subscription.
                    paypal_request_id -- str -- The server stores keys for 72
                        hours.
                    body -- CaptureSubscriptionRequest -- The request body
                        parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `200 OK` status code and a
                JSON response body that shows subscription details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/subscriptions/{id}/capture')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('paypal_request_id', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(SubscriptionTransactionDetails.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Bad Request. Request is not well-formed, syntactically incorrect, or violates schema.', SubscriptionErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()

    def list_subscription_transactions(self,
                                       options=dict()):
        """Does a GET request to /v1/billing/subscriptions/{id}/transactions.

        Lists transactions for a subscription.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the subscription.
                    start_time -- str -- The start time of the range of
                        transactions to list.
                    end_time -- str -- The end time of the range of
                        transactions to list.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `200 OK` status code and a
                JSON response body that shows subscription details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/billing/subscriptions/{id}/transactions')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .query_param(Parameter()
                         .key('start_time')
                         .value(options.get('start_time', None)))
            .query_param(Parameter()
                         .key('end_time')
                         .value(options.get('end_time', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(TransactionsList.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Bad Request. Request is not well-formed, syntactically incorrect, or violates schema.', SubscriptionErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', SubscriptionErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', SubscriptionErrorException)
            .local_error('404', 'The specified resource does not exist.', SubscriptionErrorException)
            .local_error('500', 'An internal server error has occurred.', SubscriptionErrorException)
            .local_error('default', 'The error response.', SubscriptionErrorException)
        ).execute()



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class DisputeCategory(object):

    """Implementation of the 'Dispute Category' enum.

    The condition that is covered for the transaction.

    Attributes:
        ITEM_NOT_RECEIVED: The payer paid for an item that they did not
            receive.
        UNAUTHORIZED_TRANSACTION: The payer did not authorize the payment.

    """
    ITEM_NOT_RECEIVED = 'ITEM_NOT_RECEIVED'

    UNAUTHORIZED_TRANSACTION = 'UNAUTHORIZED_TRANSACTION'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class AvsCode(object):

    """Implementation of the 'AVS Code' enum.

    The address verification code for Visa, Discover, Mastercard, or American
    Express transactions.

    Attributes:
        AVS_A: For Visa, Mastercard, or Discover transactions, the address
            matches but the zip code does not match. For American Express
            transactions, the card holder address is correct.
        AVS_B: For Visa, Mastercard, or Discover transactions, the address
            matches. International A.
        AVS_C: For Visa, Mastercard, or Discover transactions, no values
            match. International N.
        AVS_D: For Visa, Mastercard, or Discover transactions, the address and
            postal code match. International X.
        AVS_E: For Visa, Mastercard, or Discover transactions, not allowed for
            Internet or phone transactions. For American Express card holder,
            the name is incorrect but the address and postal code match.
        AVS_F: For Visa, Mastercard, or Discover transactions, the address and
            postal code match. UK-specific X. For American Express card
            holder, the name is incorrect but the address matches.
        AVS_G: For Visa, Mastercard, or Discover transactions, global is
            unavailable. Nothing matches.
        AVS_I: For Visa, Mastercard, or Discover transactions, international
            is unavailable. Not applicable.
        AVS_M: For Visa, Mastercard, or Discover transactions, the address and
            postal code match. For American Express card holder, the name,
            address, and postal code match.
        AVS_N: For Visa, Mastercard, or Discover transactions, nothing
            matches. For American Express card holder, the address and postal
            code are both incorrect.
        AVS_P: For Visa, Mastercard, or Discover transactions, postal
            international Z. Postal code only.
        AVS_R: For Visa, Mastercard, or Discover transactions, re-try the
            request. For American Express, the system is unavailable.
        AVS_S: For Visa, Mastercard, Discover, or American Express, the
            service is not supported.
        AVS_U: For Visa, Mastercard, or Discover transactions, the service is
            unavailable. For American Express, information is not available.
            For Maestro, the address is not checked or the acquirer had no
            response. The service is not available.
        AVS_W: For Visa, Mastercard, or Discover transactions, whole ZIP code.
            For American Express, the card holder name, address, and postal
            code are all incorrect.
        AVS_X: For Visa, Mastercard, or Discover transactions, exact match of
            the address and the nine-digit ZIP code. For American Express, the
            card holder name, address, and postal code are all incorrect.
        AVS_Y: For Visa, Mastercard, or Discover transactions, the address and
            five-digit ZIP code match. For American Express, the card holder
            address and postal code are both correct.
        AVS_Z: For Visa, Mastercard, or Discover transactions, the five-digit
            ZIP code matches but no address. For American Express, only the
            card holder postal code is correct.
        AVS_NULL: For Maestro, no AVS response was obtained.
        AVS_0: For Maestro, all address information matches.
        AVS_1: For Maestro, none of the address information matches.
        AVS_2: For Maestro, part of the address information matches.
        AVS_3: For Maestro, the merchant did not provide AVS information. It
            was not processed.
        AVS_4: For Maestro, the address was not checked or the acquirer had no
            response. The service is not available.

    """
    AVS_A = 'A'

    AVS_B = 'B'

    AVS_C = 'C'

    AVS_D = 'D'

    AVS_E = 'E'

    AVS_F = 'F'

    AVS_G = 'G'

    AVS_I = 'I'

    AVS_M = 'M'

    AVS_N = 'N'

    AVS_P = 'P'

    AVS_R = 'R'

    AVS_S = 'S'

    AVS_U = 'U'

    AVS_W = 'W'

    AVS_X = 'X'

    AVS_Y = 'Y'

    AVS_Z = 'Z'

    AVS_NULL = 'Null'

    AVS_0 = '0'

    AVS_1 = '1'

    AVS_2 = '2'

    AVS_3 = '3'

    AVS_4 = '4'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PaypalPaymentTokenCustomerType(object):

    """Implementation of the 'PayPal Payment Token Customer Type' enum.

    The customer type associated with the PayPal payment token. This is to
    indicate whether the customer acting on the merchant / platform is either
    a business or a consumer., The customer type associated with a digital
    wallet payment token. This is to indicate whether the customer acting on
    the merchant / platform is either a business or a consumer.

    Attributes:
        CONSUMER: The customer vaulting the PayPal payment token is a consumer
            on the merchant / platform.
        BUSINESS: The customer vaulting the PayPal payment token is a business
            on merchant / platform.

    """
    CONSUMER = 'CONSUMER'

    BUSINESS = 'BUSINESS'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class PayerName(object):

    """Implementation of the 'Payer Name' model.

    The name of the party.

    Attributes:
        prefix (str): The prefix, or title, to the party's name.
        given_name (str): When the party is a person, the party's given, or
            first, name.
        surname (str): When the party is a person, the party's surname or
            family name. Also known as the last name. Required when the party
            is a person. Use also to store multiple surnames including the
            matronymic, or mother's, surname.
        middle_name (str): When the party is a person, the party's middle
            name. Use also to store multiple middle names including the
            patronymic, or father's, middle name.
        suffix (str): The suffix for the party's name.
        alternate_full_name (str): DEPRECATED. The party's alternate name. Can
            be a business name, nickname, or any other name that cannot be
            split into first, last name. Required when the party is a business.
        full_name (str): When the party is a person, the party's full name.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "prefix": 'prefix',
        "given_name": 'given_name',
        "surname": 'surname',
        "middle_name": 'middle_name',
        "suffix": 'suffix',
        "alternate_full_name": 'alternate_full_name',
        "full_name": 'full_name'
    }

    _optionals = [
        'prefix',
        'given_name',
        'surname',
        'middle_name',
        'suffix',
        'alternate_full_name',
        'full_name',
    ]

    def __init__(self,
                 prefix=APIHelper.SKIP,
                 given_name=APIHelper.SKIP,
                 surname=APIHelper.SKIP,
                 middle_name=APIHelper.SKIP,
                 suffix=APIHelper.SKIP,
                 alternate_full_name=APIHelper.SKIP,
                 full_name=APIHelper.SKIP):
        """Constructor for the PayerName class"""

        # Initialize members of the class
        if prefix is not APIHelper.SKIP:
            self.prefix = prefix 
        if given_name is not APIHelper.SKIP:
            self.given_name = given_name 
        if surname is not APIHelper.SKIP:
            self.surname = surname 
        if middle_name is not APIHelper.SKIP:
            self.middle_name = middle_name 
        if suffix is not APIHelper.SKIP:
            self.suffix = suffix 
        if alternate_full_name is not APIHelper.SKIP:
            self.alternate_full_name = alternate_full_name 
        if full_name is not APIHelper.SKIP:
            self.full_name = full_name 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        prefix = dictionary.get("prefix") if dictionary.get("prefix") else APIHelper.SKIP
        given_name = dictionary.get("given_name") if dictionary.get("given_name") else APIHelper.SKIP
        surname = dictionary.get("surname") if dictionary.get("surname") else APIHelper.SKIP
        middle_name = dictionary.get("middle_name") if dictionary.get("middle_name") else APIHelper.SKIP
        suffix = dictionary.get("suffix") if dictionary.get("suffix") else APIHelper.SKIP
        alternate_full_name = dictionary.get("alternate_full_name") if dictionary.get("alternate_full_name") else APIHelper.SKIP
        full_name = dictionary.get("full_name") if dictionary.get("full_name") else APIHelper.SKIP
        # Return an object of this model
        return cls(prefix,
                   given_name,
                   surname,
                   middle_name,
                   suffix,
                   alternate_full_name,
                   full_name)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'prefix={(self.prefix if hasattr(self, "prefix") else None)!r}, '
                f'given_name={(self.given_name if hasattr(self, "given_name") else None)!r}, '
                f'surname={(self.surname if hasattr(self, "surname") else None)!r}, '
                f'middle_name={(self.middle_name if hasattr(self, "middle_name") else None)!r}, '
                f'suffix={(self.suffix if hasattr(self, "suffix") else None)!r}, '
                f'alternate_full_name={(self.alternate_full_name if hasattr(self, "alternate_full_name") else None)!r}, '
                f'full_name={(self.full_name if hasattr(self, "full_name") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'prefix={(self.prefix if hasattr(self, "prefix") else None)!s}, '
                f'given_name={(self.given_name if hasattr(self, "given_name") else None)!s}, '
                f'surname={(self.surname if hasattr(self, "surname") else None)!s}, '
                f'middle_name={(self.middle_name if hasattr(self, "middle_name") else None)!s}, '
                f'suffix={(self.suffix if hasattr(self, "suffix") else None)!s}, '
                f'alternate_full_name={(self.alternate_full_name if hasattr(self, "alternate_full_name") else None)!s}, '
                f'full_name={(self.full_name if hasattr(self, "full_name") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PhoneNumberWithCountryCode(object):

    """Implementation of the 'Phone Number With Country Code' model.

    The phone number in its canonical international [E.164 numbering plan
    format](https://www.itu.int/rec/T-REC-E.164/en)., The phone number, in its
    canonical international [E.164 numbering plan
    format](https://www.itu.int/rec/T-REC-E.164/en)., The phone number, in its
    canonical international [E.164 numbering plan
    format](https://www.itu.int/rec/T-REC-E.164/en).

    Attributes:
        country_code (str): The country calling code (CC), in its canonical
            international [E.164 numbering plan
            format](https://www.itu.int/rec/T-REC-E.164/en). The combined
            length of the CC and the national number must not be greater than
            15 digits. The national number consists of a national destination
            code (NDC) and subscriber number (SN).
        national_number (str): The national number, in its canonical
            international [E.164 numbering plan
            format](https://www.itu.int/rec/T-REC-E.164/en). The combined
            length of the country calling code (CC) and the national number
            must not be greater than 15 digits. The national number consists
            of a national destination code (NDC) and subscriber number (SN).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "country_code": 'country_code',
        "national_number": 'national_number'
    }

    def __init__(self,
                 country_code=None,
                 national_number=None):
        """Constructor for the PhoneNumberWithCountryCode class"""

        # Initialize members of the class
        self.country_code = country_code 
        self.national_number = national_number 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        country_code = dictionary.get("country_code") if dictionary.get("country_code") else None
        national_number = dictionary.get("national_number") if dictionary.get("national_number") else None
        # Return an object of this model
        return cls(country_code,
                   national_number)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'country_code={self.country_code!r}, '
                f'national_number={self.national_number!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'country_code={self.country_code!s}, '
                f'national_number={self.national_number!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money


class SubscriptionAmountWithBreakdown(object):

    """Implementation of the 'Subscription Amount with Breakdown' model.

    The breakdown details for the amount. Includes the gross, tax, fee, and
    shipping amounts.

    Attributes:
        gross_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        total_item_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        fee_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        shipping_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        tax_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        net_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "gross_amount": 'gross_amount',
        "total_item_amount": 'total_item_amount',
        "fee_amount": 'fee_amount',
        "shipping_amount": 'shipping_amount',
        "tax_amount": 'tax_amount',
        "net_amount": 'net_amount'
    }

    _optionals = [
        'total_item_amount',
        'fee_amount',
        'shipping_amount',
        'tax_amount',
        'net_amount',
    ]

    def __init__(self,
                 gross_amount=None,
                 total_item_amount=APIHelper.SKIP,
                 fee_amount=APIHelper.SKIP,
                 shipping_amount=APIHelper.SKIP,
                 tax_amount=APIHelper.SKIP,
                 net_amount=APIHelper.SKIP):
        """Constructor for the SubscriptionAmountWithBreakdown class"""

        # Initialize members of the class
        self.gross_amount = gross_amount 
        if total_item_amount is not APIHelper.SKIP:
            self.total_item_amount = total_item_amount 
        if fee_amount is not APIHelper.SKIP:
            self.fee_amount = fee_amount 
        if shipping_amount is not APIHelper.SKIP:
            self.shipping_amount = shipping_amount 
        if tax_amount is not APIHelper.SKIP:
            self.tax_amount = tax_amount 
        if net_amount is not APIHelper.SKIP:
            self.net_amount = net_amount 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        gross_amount = Money.from_dictionary(dictionary.get('gross_amount')) if dictionary.get('gross_amount') else None
        total_item_amount = Money.from_dictionary(dictionary.get('total_item_amount')) if 'total_item_amount' in dictionary.keys() else APIHelper.SKIP
        fee_amount = Money.from_dictionary(dictionary.get('fee_amount')) if 'fee_amount' in dictionary.keys() else APIHelper.SKIP
        shipping_amount = Money.from_dictionary(dictionary.get('shipping_amount')) if 'shipping_amount' in dictionary.keys() else APIHelper.SKIP
        tax_amount = Money.from_dictionary(dictionary.get('tax_amount')) if 'tax_amount' in dictionary.keys() else APIHelper.SKIP
        net_amount = Money.from_dictionary(dictionary.get('net_amount')) if 'net_amount' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(gross_amount,
                   total_item_amount,
                   fee_amount,
                   shipping_amount,
                   tax_amount,
                   net_amount)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'gross_amount={self.gross_amount!r}, '
                f'total_item_amount={(self.total_item_amount if hasattr(self, "total_item_amount") else None)!r}, '
                f'fee_amount={(self.fee_amount if hasattr(self, "fee_amount") else None)!r}, '
                f'shipping_amount={(self.shipping_amount if hasattr(self, "shipping_amount") else None)!r}, '
                f'tax_amount={(self.tax_amount if hasattr(self, "tax_amount") else None)!r}, '
                f'net_amount={(self.net_amount if hasattr(self, "net_amount") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'gross_amount={self.gross_amount!s}, '
                f'total_item_amount={(self.total_item_amount if hasattr(self, "total_item_amount") else None)!s}, '
                f'fee_amount={(self.fee_amount if hasattr(self, "fee_amount") else None)!s}, '
                f'shipping_amount={(self.shipping_amount if hasattr(self, "shipping_amount") else None)!s}, '
                f'tax_amount={(self.tax_amount if hasattr(self, "tax_amount") else None)!s}, '
                f'net_amount={(self.net_amount if hasattr(self, "net_amount") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class VenmoWalletExperienceContextShippingPreference(object):

    """Implementation of the 'Venmo Wallet Experience Context Shipping Preference' enum.

    The location from which the shipping address is derived.

    Attributes:
        GET_FROM_FILE: Get the customer-provided shipping address on the
            PayPal site.
        NO_SHIPPING: Redacts the shipping address from the PayPal site.
            Recommended for digital goods.
        SET_PROVIDED_ADDRESS: Get the merchant-provided address. The customer
            cannot change this address on the PayPal site. If merchant does
            not pass an address, customer can choose the address on PayPal
            pages.

    """
    GET_FROM_FILE = 'GET_FROM_FILE'

    NO_SHIPPING = 'NO_SHIPPING'

    SET_PROVIDED_ADDRESS = 'SET_PROVIDED_ADDRESS'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class UsagePattern(object):

    """Implementation of the 'Usage Pattern' enum.

    Expected business/pricing model for the billing agreement., Expected
    business/charge model for the billing agreement.

    Attributes:
        IMMEDIATE: On-demand instant payments – non-recurring, pre-paid,
            variable amount, variable frequency.
        DEFERRED: Pay after use, non-recurring post-paid, variable amount,
            irregular frequency.
        RECURRING_PREPAID: Pay upfront fixed or variable amount on a fixed
            date before the goods/service is delivered.
        RECURRING_POSTPAID: Pay on a fixed date based on usage or consumption
            after the goods/service is delivered.
        THRESHOLD_PREPAID: Charge payer when the set amount is reached or
            monthly billing cycle, whichever comes first, before the
            goods/service is delivered.
        THRESHOLD_POSTPAID: Charge payer when the set amount is reached or
            monthly billing cycle, whichever comes first, after the
            goods/service is delivered.
        SUBSCRIPTION_PREPAID: Subscription plan where the "amount due" and the
            "billing frequency" are fixed, and there is no defined duration
            with the payment due before the good/service is delivered.
        SUBSCRIPTION_POSTPAID: Subscription plan where the "amount due" and
            the "billing frequency" are fixed, and there is no defined
            duration with the payment due after the goods/services are
            delivered.
        UNSCHEDULED_PREPAID: Unscheduled card on file plan where the merchant
            can bill buyer upfront based on an agreed logic, but "amount due"
            and "frequency" can vary. Inclusive of automatic reload plans.
        UNSCHEDULED_POSTPAID: Unscheduled card on file plan where the merchant
            can bill buyer based on an agreed logic, but "amount due" and
            "frequency" can vary. Inclusive of automatic reload plans.
        INSTALLMENT_PREPAID: Merchant-managed installment plan when the
            "amount" to be paid and the "billing frequency" are fixed, but
            there is a defined number of payments with the payment due before
            the good/service is delivered.
        INSTALLMENT_POSTPAID: Merchant-managed installment plan when the
            "amount" to be paid and the "billing frequency" are fixed, but
            there is a defined number of payments with the payment due after
            the goods/services are delivered.

    """
    IMMEDIATE = 'IMMEDIATE'

    DEFERRED = 'DEFERRED'

    RECURRING_PREPAID = 'RECURRING_PREPAID'

    RECURRING_POSTPAID = 'RECURRING_POSTPAID'

    THRESHOLD_PREPAID = 'THRESHOLD_PREPAID'

    THRESHOLD_POSTPAID = 'THRESHOLD_POSTPAID'

    SUBSCRIPTION_PREPAID = 'SUBSCRIPTION_PREPAID'

    SUBSCRIPTION_POSTPAID = 'SUBSCRIPTION_POSTPAID'

    UNSCHEDULED_PREPAID = 'UNSCHEDULED_PREPAID'

    UNSCHEDULED_POSTPAID = 'UNSCHEDULED_POSTPAID'

    INSTALLMENT_PREPAID = 'INSTALLMENT_PREPAID'

    INSTALLMENT_POSTPAID = 'INSTALLMENT_POSTPAID'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class ThreeDSecureCardAuthenticationResponse(object):

    """Implementation of the 'Three-D Secure Card Authentication Response' model.

    Results of 3D Secure Authentication.

    Attributes:
        authentication_status (PaResStatus): Transactions status result
            identifier. The outcome of the issuer's authentication.
        enrollment_status (EnrollmentStatus): Status of Authentication
            eligibility.
        authentication_id (str): The externally received 3ds authentication
            id, to be returned in card detokenization response.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "authentication_status": 'authentication_status',
        "enrollment_status": 'enrollment_status',
        "authentication_id": 'authentication_id'
    }

    _optionals = [
        'authentication_status',
        'enrollment_status',
        'authentication_id',
    ]

    def __init__(self,
                 authentication_status=APIHelper.SKIP,
                 enrollment_status=APIHelper.SKIP,
                 authentication_id=APIHelper.SKIP):
        """Constructor for the ThreeDSecureCardAuthenticationResponse class"""

        # Initialize members of the class
        if authentication_status is not APIHelper.SKIP:
            self.authentication_status = authentication_status 
        if enrollment_status is not APIHelper.SKIP:
            self.enrollment_status = enrollment_status 
        if authentication_id is not APIHelper.SKIP:
            self.authentication_id = authentication_id 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        authentication_status = dictionary.get("authentication_status") if dictionary.get("authentication_status") else APIHelper.SKIP
        enrollment_status = dictionary.get("enrollment_status") if dictionary.get("enrollment_status") else APIHelper.SKIP
        authentication_id = dictionary.get("authentication_id") if dictionary.get("authentication_id") else APIHelper.SKIP
        # Return an object of this model
        return cls(authentication_status,
                   enrollment_status,
                   authentication_id)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'authentication_status={(self.authentication_status if hasattr(self, "authentication_status") else None)!r}, '
                f'enrollment_status={(self.enrollment_status if hasattr(self, "enrollment_status") else None)!r}, '
                f'authentication_id={(self.authentication_id if hasattr(self, "authentication_id") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'authentication_status={(self.authentication_status if hasattr(self, "authentication_status") else None)!s}, '
                f'enrollment_status={(self.enrollment_status if hasattr(self, "enrollment_status") else None)!s}, '
                f'authentication_id={(self.authentication_id if hasattr(self, "authentication_id") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class OsType(object):

    """Implementation of the 'OS Type' enum.

    Operating System type of the device that the buyer is using.

    Attributes:
        ANDROID: Google Android OS.
        IOS: Apple OS typically found in Apple mobile devices.
        OTHER: Any other OS type.

    """
    ANDROID = 'ANDROID'

    IOS = 'IOS'

    OTHER = 'OTHER'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class VenmoWalletExperienceContextUserAction(object):

    """Implementation of the 'Venmo Wallet Experience Context User Action' enum.

    Configures a Continue or Pay Now checkout flow.

    Attributes:
        CONTINUE: After you redirect the customer to the Venmo payment page, a
            Continue button appears. Use this option when the final amount is
            not known when the checkout flow is initiated and you want to
            redirect the customer to the merchant page without processing the
            payment.
        PAY_NOW: After you redirect the customer to the Venmo payment page, a
            Pay Now button appears. Use this option when the final amount is
            known when the checkout is initiated and you want to process the
            payment immediately when the customer clicks Pay Now.

    """
    CONTINUE = 'CONTINUE'

    PAY_NOW = 'PAY_NOW'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class OrderApplicationContextUserAction(object):

    """Implementation of the 'Order Application Context User Action' enum.

    DEPRECATED. Configures a Continue or Pay Now checkout flow.  The fields in
    `application_context` are now available in the `experience_context` object
    under the `payment_source` which supports them (eg.
    `payment_source.paypal.experience_context.user_action`). Please specify
    this field in the `experience_context` object instead of the
    `application_context` object.

    Attributes:
        CONTINUE: After you redirect the customer to the PayPal payment page,
            a Continue button appears. Use this option when the final amount
            is not known when the checkout flow is initiated and you want to
            redirect the customer to the merchant page without processing the
            payment.
        PAY_NOW: After you redirect the customer to the PayPal payment page, a
            Pay Now button appears. Use this option when the final amount is
            known when the checkout is initiated and you want to process the
            payment immediately when the customer clicks Pay Now.

    """
    CONTINUE = 'CONTINUE'

    PAY_NOW = 'PAY_NOW'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.item_details import ItemDetails


class CartInformation(object):

    """Implementation of the 'Cart Information' model.

    The cart information.

    Attributes:
        item_details (List[ItemDetails]): An array of item details.
        tax_inclusive (bool): Indicates whether the item amount or the
            shipping amount already includes tax.
        paypal_invoice_id (str): The ID of the invoice. Appears for only
            PayPal-generated invoices.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "item_details": 'item_details',
        "tax_inclusive": 'tax_inclusive',
        "paypal_invoice_id": 'paypal_invoice_id'
    }

    _optionals = [
        'item_details',
        'tax_inclusive',
        'paypal_invoice_id',
    ]

    def __init__(self,
                 item_details=APIHelper.SKIP,
                 tax_inclusive=False,
                 paypal_invoice_id=APIHelper.SKIP):
        """Constructor for the CartInformation class"""

        # Initialize members of the class
        if item_details is not APIHelper.SKIP:
            self.item_details = item_details 
        self.tax_inclusive = tax_inclusive 
        if paypal_invoice_id is not APIHelper.SKIP:
            self.paypal_invoice_id = paypal_invoice_id 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        item_details = None
        if dictionary.get('item_details') is not None:
            item_details = [ItemDetails.from_dictionary(x) for x in dictionary.get('item_details')]
        else:
            item_details = APIHelper.SKIP
        tax_inclusive = dictionary.get("tax_inclusive") if dictionary.get("tax_inclusive") else False
        paypal_invoice_id = dictionary.get("paypal_invoice_id") if dictionary.get("paypal_invoice_id") else APIHelper.SKIP
        # Return an object of this model
        return cls(item_details,
                   tax_inclusive,
                   paypal_invoice_id)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'item_details={(self.item_details if hasattr(self, "item_details") else None)!r}, '
                f'tax_inclusive={(self.tax_inclusive if hasattr(self, "tax_inclusive") else None)!r}, '
                f'paypal_invoice_id={(self.paypal_invoice_id if hasattr(self, "paypal_invoice_id") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'item_details={(self.item_details if hasattr(self, "item_details") else None)!s}, '
                f'tax_inclusive={(self.tax_inclusive if hasattr(self, "tax_inclusive") else None)!s}, '
                f'paypal_invoice_id={(self.paypal_invoice_id if hasattr(self, "paypal_invoice_id") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.callback_configuration import CallbackConfiguration


class VenmoWalletExperienceContext(object):

    """Implementation of the 'Venmo Wallet Experience Context' model.

    Customizes the buyer experience during the approval process for payment
    with Venmo. Note: Partners and Marketplaces might configure
    shipping_preference during partner account setup, which overrides the
    request values.

    Attributes:
        brand_name (str): The business name of the merchant. The pattern is
            defined by an external party and supports Unicode.
        shipping_preference (VenmoWalletExperienceContextShippingPreference):
            The location from which the shipping address is derived.
        order_update_callback_config (CallbackConfiguration): CallBack
            Configuration that the merchant can provide to PayPal/Venmo.
        user_action (VenmoWalletExperienceContextUserAction): Configures a
            Continue or Pay Now checkout flow.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "brand_name": 'brand_name',
        "shipping_preference": 'shipping_preference',
        "order_update_callback_config": 'order_update_callback_config',
        "user_action": 'user_action'
    }

    _optionals = [
        'brand_name',
        'shipping_preference',
        'order_update_callback_config',
        'user_action',
    ]

    def __init__(self,
                 brand_name=APIHelper.SKIP,
                 shipping_preference='GET_FROM_FILE',
                 order_update_callback_config=APIHelper.SKIP,
                 user_action='CONTINUE'):
        """Constructor for the VenmoWalletExperienceContext class"""

        # Initialize members of the class
        if brand_name is not APIHelper.SKIP:
            self.brand_name = brand_name 
        self.shipping_preference = shipping_preference 
        if order_update_callback_config is not APIHelper.SKIP:
            self.order_update_callback_config = order_update_callback_config 
        self.user_action = user_action 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        brand_name = dictionary.get("brand_name") if dictionary.get("brand_name") else APIHelper.SKIP
        shipping_preference = dictionary.get("shipping_preference") if dictionary.get("shipping_preference") else 'GET_FROM_FILE'
        order_update_callback_config = CallbackConfiguration.from_dictionary(dictionary.get('order_update_callback_config')) if 'order_update_callback_config' in dictionary.keys() else APIHelper.SKIP
        user_action = dictionary.get("user_action") if dictionary.get("user_action") else 'CONTINUE'
        # Return an object of this model
        return cls(brand_name,
                   shipping_preference,
                   order_update_callback_config,
                   user_action)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'brand_name={(self.brand_name if hasattr(self, "brand_name") else None)!r}, '
                f'shipping_preference={(self.shipping_preference if hasattr(self, "shipping_preference") else None)!r}, '
                f'order_update_callback_config={(self.order_update_callback_config if hasattr(self, "order_update_callback_config") else None)!r}, '
                f'user_action={(self.user_action if hasattr(self, "user_action") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'brand_name={(self.brand_name if hasattr(self, "brand_name") else None)!s}, '
                f'shipping_preference={(self.shipping_preference if hasattr(self, "shipping_preference") else None)!s}, '
                f'order_update_callback_config={(self.order_update_callback_config if hasattr(self, "order_update_callback_config") else None)!s}, '
                f'user_action={(self.user_action if hasattr(self, "user_action") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class CheckoutPaymentIntent(object):

    """Implementation of the 'Checkout Payment Intent' enum.

    The intent to either capture payment immediately or authorize a payment
    for an order after order creation.

    Attributes:
        CAPTURE: The merchant intends to capture payment immediately after the
            customer makes a payment.
        AUTHORIZE: The merchant intends to authorize a payment and place funds
            on hold after the customer makes a payment. Authorized payments
            are best captured within three days of authorization but are
            available to capture for up to 29 days. After the three-day honor
            period, the original authorized payment expires and you must
            re-authorize the payment. You must make a separate request to
            capture payments on demand. This intent is not supported when you
            have more than one `purchase_unit` within your order.

    """
    CAPTURE = 'CAPTURE'

    AUTHORIZE = 'AUTHORIZE'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class GooglePayAuthenticationMethod(object):

    """Implementation of the 'Google Pay Authentication Method' enum.

    Authentication Method which is used for the card transaction.

    Attributes:
        PAN_ONLY: This authentication method is associated with payment cards
            stored on file with the user's Google Account. Returned payment
            data includes primary account number (PAN) with the expiration
            month and the expiration year.
        CRYPTOGRAM_3DS: Returned payment data includes a 3-D Secure (3DS)
            cryptogram generated on the device. -> If
            authentication_method=CRYPTOGRAM, it is required that 'cryptogram'
            parameter in the request has a valid 3-D Secure (3DS) cryptogram
            generated on the device.

    """
    PAN_ONLY = 'PAN_ONLY'

    CRYPTOGRAM_3DS = 'CRYPTOGRAM_3DS'

